@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1996,2008 Oracle.  All rights reserved.
  *
- * $Id: db_upgrade.c,v 12.9 2006/08/26 09:23:22 bostic Exp $
+ * $Id: db_upgrade.c,v 12.13 2008/01/08 20:58:16 bostic Exp $
  */
 
 #include "db_config.h"
@@ -13,7 +12,7 @@
 
 #ifndef lint
 static const char copyright[] =
-    "Copyright (c) 1996-2006\nOracle Corporation.  All rights reserved.\n";
+    "Copyright (c) 1996,2008 Oracle.  All rights reserved.\n";
 #endif
 
 int db_upgrade_main __P((int, char *[]));
@@ -49,7 +48,7 @@ db_upgrade_main(argc, argv)
 	int ch, exitval, nflag, ret, t_ret, verbose;
 	char *home, *passwd;
 
-	if ((progname = strrchr(argv[0], '/')) == NULL)
+	if ((progname = __db_rpath(argv[0])) == NULL)
 		progname = argv[0];
 	else
 		++progname;
