@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996,2007 Oracle.  All rights reserved.
+ * Copyright (c) 1996,2008 Oracle.  All rights reserved.
  *
- * $Id: snprintf.c,v 12.7 2007/05/17 15:14:54 bostic Exp $
+ * $Id: snprintf.c,v 12.9 2008/01/08 20:58:08 bostic Exp $
  */
 
 #include "db_config.h"
@@ -119,7 +119,7 @@ sprintf_overflow()
 	exit(1);
 
 	/* But firm. */
-	__os_abort();
+	__os_abort(NULL);
 
 	/* NOTREACHED */
 }
