@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1996,2008 Oracle.  All rights reserved.
  *
- * $Id: db_byteorder.c,v 12.5 2006/08/24 14:45:10 bostic Exp $
+ * $Id: db_byteorder.c,v 12.10 2008/01/08 20:58:08 bostic Exp $
  */
 
 #include "db_config.h"
@@ -37,30 +36,26 @@ __db_isbigendian()
  *	Return if we need to do byte swapping, checking for illegal
  *	values.
  *
- * PUBLIC: int __db_byteorder __P((DB_ENV *, int));
+ * PUBLIC: int __db_byteorder __P((ENV *, int));
  */
 int
-__db_byteorder(dbenv, lorder)
-	DB_ENV *dbenv;
+__db_byteorder(env, lorder)
+	ENV *env;
 	int lorder;
 {
-	int is_bigendian;
-
-	is_bigendian = __db_isbigendian();
-
 	switch (lorder) {
 	case 0:
 		break;
 	case 1234:
-		if (is_bigendian)
+		if (!F_ISSET(env, ENV_LITTLEENDIAN))
 			return (DB_SWAPBYTES);
 		break;
 	case 4321:
-		if (!is_bigendian)
+		if (F_ISSET(env, ENV_LITTLEENDIAN))
 			return (DB_SWAPBYTES);
 		break;
 	default:
-		__db_errx(dbenv,
+		__db_errx(env,
 	    "unsupported byte order, only big and little-endian supported");
 		return (EINVAL);
 	}
