@@ -1,5 +1,5 @@
 /*
- * $Id: mt19937db.c,v 12.6 2007/04/18 18:16:04 bostic Exp $
+ * $Id: mt19937db.c,v 12.9 2008/03/12 19:23:38 mbrey Exp $
  */
 #include "db_config.h"
 
@@ -34,59 +34,59 @@
 /* Vol. 8, No. 1, January 1998, pp 3--30.                          */
 
 /* Period parameters */
-#define N 624
-#define M 397
-#define MATRIX_A 0x9908b0df   /* constant vector a */
-#define UPPER_MASK 0x80000000 /* most significant w-r bits */
-#define LOWER_MASK 0x7fffffff /* least significant r bits */
+#define	N 624
+#define	M 397
+#define	MATRIX_A 0x9908b0df   /* constant vector a */
+#define	UPPER_MASK 0x80000000 /* most significant w-r bits */
+#define	LOWER_MASK 0x7fffffff /* least significant r bits */
 
 /* Tempering parameters */
-#define TEMPERING_MASK_B 0x9d2c5680
-#define TEMPERING_MASK_C 0xefc60000
-#define TEMPERING_SHIFT_U(y)  (y >> 11)
-#define TEMPERING_SHIFT_S(y)  (y << 7)
-#define TEMPERING_SHIFT_T(y)  (y << 15)
-#define TEMPERING_SHIFT_L(y)  (y >> 18)
+#define	TEMPERING_MASK_B 0x9d2c5680
+#define	TEMPERING_MASK_C 0xefc60000
+#define	TEMPERING_SHIFT_U(y)  (y >> 11)
+#define	TEMPERING_SHIFT_S(y)  (y << 7)
+#define	TEMPERING_SHIFT_T(y)  (y << 15)
+#define	TEMPERING_SHIFT_L(y)  (y >> 18)
 
 static void __db_sgenrand __P((unsigned long, unsigned long *, int *));
 #ifdef	NOT_USED
 static void __db_lsgenrand __P((unsigned long *, unsigned long *, int *));
 #endif
-static unsigned long __db_genrand __P((DB_ENV *));
+static unsigned long __db_genrand __P((ENV *));
 
 /*
  * __db_generate_iv --
  *	Generate an initialization vector (IV)
  *
- * PUBLIC: int __db_generate_iv __P((DB_ENV *, u_int32_t *));
+ * PUBLIC: int __db_generate_iv __P((ENV *, u_int32_t *));
  */
 int
-__db_generate_iv(dbenv, iv)
-	DB_ENV *dbenv;
+__db_generate_iv(env, iv)
+	ENV *env;
 	u_int32_t *iv;
 {
 	int i, n, ret;
 
 	ret = 0;
 	n = DB_IV_BYTES / sizeof(u_int32_t);
-	MUTEX_LOCK(dbenv, dbenv->mtx_mt);
-	if (dbenv->mt == NULL) {
-		if ((ret = __os_calloc(dbenv, 1, N*sizeof(unsigned long),
-		    &dbenv->mt)) != 0)
+	MUTEX_LOCK(env, env->mtx_mt);
+	if (env->mt == NULL) {
+		if ((ret = __os_calloc(env, 1, N*sizeof(unsigned long),
+		    &env->mt)) != 0)
 			return (ret);
 		/* mti==N+1 means mt[N] is not initialized */
-		dbenv->mti = N + 1;
+		env->mti = N + 1;
 	}
 	for (i = 0; i < n; i++) {
 		/*
 		 * We do not allow 0.  If we get one just try again.
 		 */
 		do {
-			iv[i] = (u_int32_t)__db_genrand(dbenv);
+			iv[i] = (u_int32_t)__db_genrand(env);
 		} while (iv[i] == 0);
 	}
 
-	MUTEX_UNLOCK(dbenv, dbenv->mtx_mt);
+	MUTEX_UNLOCK(env, env->mtx_mt);
 	return (0);
 }
 
@@ -101,10 +101,10 @@ __db_sgenrand(seed, mt, mtip)
 
     DB_ASSERT(NULL, seed != 0);
     for (i=0;i<N;i++) {
-         mt[i] = seed & 0xffff0000;
-         seed = 69069 * seed + 1;
-         mt[i] |= (seed & 0xffff0000) >> 16;
-         seed = 69069 * seed + 1;
+	 mt[i] = seed & 0xffff0000;
+	 seed = 69069 * seed + 1;
+	 mt[i] |= (seed & 0xffff0000) >> 16;
+	 seed = 69069 * seed + 1;
     }
     *mtip = N;
 }
@@ -135,8 +135,8 @@ __db_lsgenrand(seed_array, mt, mtip)
 #endif
 
 static unsigned long
-__db_genrand(dbenv)
-    DB_ENV *dbenv;
+__db_genrand(env)
+    ENV *env;
 {
     db_timespec ts;
     unsigned long y;
@@ -145,39 +145,39 @@ __db_genrand(dbenv)
     u_int32_t seed;
 
     /*
-     * We are called with DB_ENV->mtx_mt locked.
+     * We are called with ENV->mtx_mt locked.
      */
-    if (dbenv->mti >= N) { /* generate N words at one time */
-        int kk;
+    if (env->mti >= N) { /* generate N words at one time */
+	int kk;
 
-        if (dbenv->mti == N+1) {  /* if sgenrand() has not been called, */
+	if (env->mti == N+1) {  /* if sgenrand() has not been called, */
 		/*
 		 * Seed the generator with the hashed time.  The __db_mac
 		 * function will return 4 bytes if we don't send in a key.
 		 */
 		do {
-			__os_gettime(dbenv, &ts);
+			__os_gettime(env, &ts, 1);
 			__db_chksum(NULL, (u_int8_t *)&ts.tv_sec,
 			    sizeof(ts.tv_sec), NULL, (u_int8_t *)&seed);
 		} while (seed == 0);
-        	__db_sgenrand((unsigned long)seed, dbenv->mt, &dbenv->mti);
+		__db_sgenrand((unsigned long)seed, env->mt, &env->mti);
 	}
 
-        for (kk=0;kk<N-M;kk++) {
-            y = (dbenv->mt[kk]&UPPER_MASK)|(dbenv->mt[kk+1]&LOWER_MASK);
-            dbenv->mt[kk] = dbenv->mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1];
-        }
-        for (;kk<N-1;kk++) {
-            y = (dbenv->mt[kk]&UPPER_MASK)|(dbenv->mt[kk+1]&LOWER_MASK);
-            dbenv->mt[kk] = dbenv->mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1];
-        }
-        y = (dbenv->mt[N-1]&UPPER_MASK)|(dbenv->mt[0]&LOWER_MASK);
-        dbenv->mt[N-1] = dbenv->mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1];
+	for (kk=0;kk<N-M;kk++) {
+	    y = (env->mt[kk]&UPPER_MASK)|(env->mt[kk+1]&LOWER_MASK);
+	    env->mt[kk] = env->mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1];
+	}
+	for (;kk<N-1;kk++) {
+	    y = (env->mt[kk]&UPPER_MASK)|(env->mt[kk+1]&LOWER_MASK);
+	    env->mt[kk] = env->mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1];
+	}
+	y = (env->mt[N-1]&UPPER_MASK)|(env->mt[0]&LOWER_MASK);
+	env->mt[N-1] = env->mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1];
 
-        dbenv->mti = 0;
+	env->mti = 0;
     }
 
-    y = dbenv->mt[dbenv->mti++];
+    y = env->mt[env->mti++];
     y ^= TEMPERING_SHIFT_U(y);
     y ^= TEMPERING_SHIFT_S(y) & TEMPERING_MASK_B;
     y ^= TEMPERING_SHIFT_T(y) & TEMPERING_MASK_C;
