@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1997,2008 Oracle.  All rights reserved.
  *
- * $Id: cxx_mpool.cpp,v 12.5 2006/08/24 14:45:13 bostic Exp $
+ * $Id: cxx_mpool.cpp,v 12.12 2008/01/08 20:58:09 bostic Exp $
  */
 
 #include "db_config.h"
@@ -31,7 +30,7 @@ int DbMpoolFile::_name _argspec						\
 	else								\
 		ret = mpf->_name _arglist;				\
 	if (!_retok(ret))						\
-		DB_ERROR(DbEnv::get_DbEnv(mpf->dbenv), \
+		DB_ERROR(DbEnv::get_DbEnv(mpf->env->dbenv), 		\
 			"DbMpoolFile::"#_name, ret, ON_ERROR_UNKNOWN);	\
 	return (ret);							\
 }
@@ -63,7 +62,7 @@ int DbMpoolFile::close(u_int32_t flags)
 {
 	DB_MPOOLFILE *mpf = unwrap(this);
 	int ret;
-	DbEnv *dbenv = DbEnv::get_DbEnv(mpf->dbenv);
+	DbEnv *dbenv = DbEnv::get_DbEnv(mpf->env->dbenv);
 
 	if (mpf == NULL)
 		ret = EINVAL;
@@ -88,10 +87,9 @@ DB_MPOOLFILE_METHOD(get,
 DB_MPOOLFILE_METHOD(open,
     (const char *file, u_int32_t flags, int mode, size_t pagesize),
     (mpf, file, flags, mode, pagesize), DB_RETOK_STD)
-DB_MPOOLFILE_METHOD(put, (void *pgaddr, u_int32_t flags),
-    (mpf, pgaddr, flags), DB_RETOK_STD)
-DB_MPOOLFILE_METHOD(set, (void *pgaddr, u_int32_t flags),
-    (mpf, pgaddr, flags), DB_RETOK_STD)
+DB_MPOOLFILE_METHOD(put,
+    (void *pgaddr, DB_CACHE_PRIORITY priority, u_int32_t flags),
+    (mpf, pgaddr, priority, flags), DB_RETOK_STD)
 DB_MPOOLFILE_METHOD(get_clear_len, (u_int32_t *lenp),
     (mpf, lenp), DB_RETOK_STD)
 DB_MPOOLFILE_METHOD(set_clear_len, (u_int32_t len),
@@ -108,6 +106,8 @@ DB_MPOOLFILE_METHOD(get_ftype, (int *ftypep),
     (mpf, ftypep), DB_RETOK_STD)
 DB_MPOOLFILE_METHOD(set_ftype, (int ftype),
     (mpf, ftype), DB_RETOK_STD)
+DB_MPOOLFILE_METHOD(get_last_pgno, (db_pgno_t *pgnop),
+    (mpf, pgnop), DB_RETOK_STD)
 DB_MPOOLFILE_METHOD(get_lsn_offset, (int32_t *offsetp),
     (mpf, offsetp), DB_RETOK_STD)
 DB_MPOOLFILE_METHOD(set_lsn_offset, (int32_t offset),
