@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2008 Oracle.  All rights reserved.
  *
- * $Id: SampleDatabase.java,v 12.6 2007/05/17 15:15:34 bostic Exp $
+ * $Id: SampleDatabase.java,v 12.8 2008/02/07 17:12:22 mark Exp $
  */
 
 package collections.ship.index;
@@ -20,6 +20,7 @@ import com.sleepycat.db.DatabaseException;
 import com.sleepycat.db.DatabaseType;
 import com.sleepycat.db.Environment;
 import com.sleepycat.db.EnvironmentConfig;
+import com.sleepycat.db.ForeignKeyDeleteAction;
 import com.sleepycat.db.SecondaryConfig;
 import com.sleepycat.db.SecondaryDatabase;
 
@@ -110,20 +111,22 @@ public class SampleDatabase {
                                          SupplierData.class,
                                          String.class));
         supplierByCityDb = env.openSecondaryDatabase(null, SUPPLIER_CITY_INDEX,
-                                                     null,
-                                                     supplierDb,
+                                                     null, supplierDb,
                                                      secConfig);
 
+        secConfig.setForeignKeyDatabase(partDb);
+        secConfig.setForeignKeyDeleteAction(ForeignKeyDeleteAction.CASCADE);
         secConfig.setKeyCreator(
             new ShipmentByPartKeyCreator(javaCatalog,
                                          ShipmentKey.class,
                                          ShipmentData.class,
                                          PartKey.class));
         shipmentByPartDb = env.openSecondaryDatabase(null, SHIPMENT_PART_INDEX,
-                                                     null,
-                                                     shipmentDb,
+                                                     null, shipmentDb,
                                                      secConfig);
 
+        secConfig.setForeignKeyDatabase(supplierDb);
+        secConfig.setForeignKeyDeleteAction(ForeignKeyDeleteAction.CASCADE);
         secConfig.setKeyCreator(
             new ShipmentBySupplierKeyCreator(javaCatalog,
                                              ShipmentKey.class,
@@ -131,8 +134,7 @@ public class SampleDatabase {
                                              SupplierKey.class));
         shipmentBySupplierDb = env.openSecondaryDatabase(null,
                                                      SHIPMENT_SUPPLIER_INDEX,
-                                                     null,
-                                                     shipmentDb,
+                                                     null, shipmentDb,
                                                      secConfig);
     }
 
