@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996,2007 Oracle.  All rights reserved.
+ * Copyright (c) 1996,2008 Oracle.  All rights reserved.
  *
- * $Id: lock_method.c,v 12.17 2007/05/17 15:15:43 bostic Exp $
+ * $Id: lock_method.c,v 12.25 2008/05/07 12:27:35 bschmeck Exp $
  */
 
 #include "db_config.h"
@@ -21,6 +21,7 @@ int
 __lock_env_create(dbenv)
 	DB_ENV *dbenv;
 {
+	u_int32_t cpu;
 	/*
 	 * !!!
 	 * Our caller has not yet had the opportunity to reset the panic
@@ -31,6 +32,15 @@ __lock_env_create(dbenv)
 	dbenv->lk_max_lockers = DB_LOCK_DEFAULT_N;
 	dbenv->lk_max_objects = DB_LOCK_DEFAULT_N;
 
+	/*
+	 * Default to 10 partitions per cpu.  This seems to be near
+	 * the point of diminishing returns on Xeon type processors.
+	 * Cpu count often returns the number of hyper threads and if
+	 * there is only one CPU you probably do not want to run partitions.
+	 */
+	cpu = __os_cpu_count();
+	dbenv->lk_partitions = cpu > 1 ? 10 * cpu : 1;
+
 	return (0);
 }
 
@@ -44,8 +54,12 @@ void
 __lock_env_destroy(dbenv)
 	DB_ENV *dbenv;
 {
+	ENV *env;
+
+	env = dbenv->env;
+
 	if (dbenv->lk_conflicts != NULL) {
-		__os_free(dbenv, dbenv->lk_conflicts);
+		__os_free(env, dbenv->lk_conflicts);
 		dbenv->lk_conflicts = NULL;
 	}
 }
@@ -64,13 +78,15 @@ __lock_get_lk_conflicts(dbenv, lk_conflictsp, lk_modesp)
 	int *lk_modesp;
 {
 	DB_LOCKTAB *lt;
+	ENV *env;
 
-	ENV_NOT_CONFIGURED(dbenv,
-	    dbenv->lk_handle, "DB_ENV->get_lk_conflicts", DB_INIT_LOCK);
+	env = dbenv->env;
+	lt = env->lk_handle;
 
-	lt = dbenv->lk_handle;
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->get_lk_conflicts", DB_INIT_LOCK);
 
-	if (LOCKING_ON(dbenv)) {
+	if (LOCKING_ON(env)) {
 		/* Cannot be set after open, no lock required to read. */
 		if (lk_conflictsp != NULL)
 			*lk_conflictsp = lt->conflicts;
@@ -98,15 +114,18 @@ __lock_set_lk_conflicts(dbenv, lk_conflicts, lk_modes)
 	u_int8_t *lk_conflicts;
 	int lk_modes;
 {
+	ENV *env;
 	int ret;
 
-	ENV_ILLEGAL_AFTER_OPEN(dbenv, "DB_ENV->set_lk_conflicts");
+	env = dbenv->env;
+
+	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_lk_conflicts");
 
 	if (dbenv->lk_conflicts != NULL) {
-		__os_free(dbenv, dbenv->lk_conflicts);
+		__os_free(env, dbenv->lk_conflicts);
 		dbenv->lk_conflicts = NULL;
 	}
-	if ((ret = __os_malloc(dbenv,
+	if ((ret = __os_malloc(env,
 	    (size_t)(lk_modes * lk_modes), &dbenv->lk_conflicts)) != 0)
 		return (ret);
 	memcpy(
@@ -125,15 +144,21 @@ __lock_get_lk_detect(dbenv, lk_detectp)
 	u_int32_t *lk_detectp;
 {
 	DB_LOCKTAB *lt;
+	DB_THREAD_INFO *ip;
+	ENV *env;
 
-	ENV_NOT_CONFIGURED(dbenv,
-	    dbenv->lk_handle, "DB_ENV->get_lk_detect", DB_INIT_LOCK);
+	env = dbenv->env;
 
-	if (LOCKING_ON(dbenv)) {
-		lt = dbenv->lk_handle;
-		LOCK_REGION_LOCK(dbenv);
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->get_lk_detect", DB_INIT_LOCK);
+
+	if (LOCKING_ON(env)) {
+		lt = env->lk_handle;
+		ENV_ENTER(env, ip);
+		LOCK_REGION_LOCK(env);
 		*lk_detectp = ((DB_LOCKREGION *)lt->reginfo.primary)->detect;
-		LOCK_REGION_UNLOCK(dbenv);
+		LOCK_REGION_UNLOCK(env);
+		ENV_LEAVE(env, ip);
 	} else
 		*lk_detectp = dbenv->lk_detect;
 	return (0);
@@ -150,12 +175,16 @@ __lock_set_lk_detect(dbenv, lk_detect)
 	DB_ENV *dbenv;
 	u_int32_t lk_detect;
 {
-	DB_LOCKTAB *lt;
 	DB_LOCKREGION *region;
+	DB_LOCKTAB *lt;
+	DB_THREAD_INFO *ip;
+	ENV *env;
 	int ret;
 
-	ENV_NOT_CONFIGURED(dbenv,
-	    dbenv->lk_handle, "DB_ENV->set_lk_detect", DB_INIT_LOCK);
+	env = dbenv->env;
+
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->set_lk_detect", DB_INIT_LOCK);
 
 	switch (lk_detect) {
 	case DB_LOCK_DEFAULT:
@@ -169,16 +198,18 @@ __lock_set_lk_detect(dbenv, lk_detect)
 	case DB_LOCK_YOUNGEST:
 		break;
 	default:
-		__db_errx(dbenv,
+		__db_errx(env,
 	    "DB_ENV->set_lk_detect: unknown deadlock detection mode specified");
 		return (EINVAL);
 	}
 
 	ret = 0;
-	if (LOCKING_ON(dbenv)) {
-		lt = dbenv->lk_handle;
+	if (LOCKING_ON(env)) {
+		ENV_ENTER(env, ip);
+
+		lt = env->lk_handle;
 		region = lt->reginfo.primary;
-		LOCK_REGION_LOCK(dbenv);
+		LOCK_REGION_LOCK(env);
 		/*
 		 * Check for incompatible automatic deadlock detection requests.
 		 * There are scenarios where changing the detector configuration
@@ -191,13 +222,14 @@ __lock_set_lk_detect(dbenv, lk_detect)
 		if (region->detect != DB_LOCK_NORUN &&
 		    lk_detect != DB_LOCK_DEFAULT &&
 		    region->detect != lk_detect) {
-			__db_errx(dbenv,
+			__db_errx(env,
 	    "DB_ENV->set_lk_detect: incompatible deadlock detector mode");
 			ret = EINVAL;
 		} else
 			if (region->detect == DB_LOCK_NORUN)
 				region->detect = lk_detect;
-		LOCK_REGION_UNLOCK(dbenv);
+		LOCK_REGION_UNLOCK(env);
+		ENV_LEAVE(env, ip);
 	} else
 		dbenv->lk_detect = lk_detect;
 
@@ -212,13 +244,17 @@ __lock_get_lk_max_locks(dbenv, lk_maxp)
 	DB_ENV *dbenv;
 	u_int32_t *lk_maxp;
 {
-	ENV_NOT_CONFIGURED(dbenv,
-	    dbenv->lk_handle, "DB_ENV->get_lk_maxlocks", DB_INIT_LOCK);
+	ENV *env;
 
-	if (LOCKING_ON(dbenv)) {
+	env = dbenv->env;
+
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->get_lk_maxlocks", DB_INIT_LOCK);
+
+	if (LOCKING_ON(env)) {
 		/* Cannot be set after open, no lock required to read. */
 		*lk_maxp = ((DB_LOCKREGION *)
-		    dbenv->lk_handle->reginfo.primary)->stat.st_maxlocks;
+		    env->lk_handle->reginfo.primary)->stat.st_maxlocks;
 	} else
 		*lk_maxp = dbenv->lk_max;
 	return (0);
@@ -235,7 +271,11 @@ __lock_set_lk_max_locks(dbenv, lk_max)
 	DB_ENV *dbenv;
 	u_int32_t lk_max;
 {
-	ENV_ILLEGAL_AFTER_OPEN(dbenv, "DB_ENV->set_lk_max_locks");
+	ENV *env;
+
+	env = dbenv->env;
+
+	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_lk_max_locks");
 
 	dbenv->lk_max = lk_max;
 	return (0);
@@ -249,13 +289,17 @@ __lock_get_lk_max_lockers(dbenv, lk_maxp)
 	DB_ENV *dbenv;
 	u_int32_t *lk_maxp;
 {
-	ENV_NOT_CONFIGURED(dbenv,
-	    dbenv->lk_handle, "DB_ENV->get_lk_max_lockers", DB_INIT_LOCK);
+	ENV *env;
 
-	if (LOCKING_ON(dbenv)) {
+	env = dbenv->env;
+
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->get_lk_max_lockers", DB_INIT_LOCK);
+
+	if (LOCKING_ON(env)) {
 		/* Cannot be set after open, no lock required to read. */
 		*lk_maxp = ((DB_LOCKREGION *)
-		    dbenv->lk_handle->reginfo.primary)->stat.st_maxlockers;
+		    env->lk_handle->reginfo.primary)->stat.st_maxlockers;
 	} else
 		*lk_maxp = dbenv->lk_max_lockers;
 	return (0);
@@ -272,7 +316,11 @@ __lock_set_lk_max_lockers(dbenv, lk_max)
 	DB_ENV *dbenv;
 	u_int32_t lk_max;
 {
-	ENV_ILLEGAL_AFTER_OPEN(dbenv, "DB_ENV->set_lk_max_lockers");
+	ENV *env;
+
+	env = dbenv->env;
+
+	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_lk_max_lockers");
 
 	dbenv->lk_max_lockers = lk_max;
 	return (0);
@@ -286,13 +334,17 @@ __lock_get_lk_max_objects(dbenv, lk_maxp)
 	DB_ENV *dbenv;
 	u_int32_t *lk_maxp;
 {
-	ENV_NOT_CONFIGURED(dbenv,
-	    dbenv->lk_handle, "DB_ENV->get_lk_max_objects", DB_INIT_LOCK);
+	ENV *env;
 
-	if (LOCKING_ON(dbenv)) {
+	env = dbenv->env;
+
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->get_lk_max_objects", DB_INIT_LOCK);
+
+	if (LOCKING_ON(env)) {
 		/* Cannot be set after open, no lock required to read. */
 		*lk_maxp = ((DB_LOCKREGION *)
-		    dbenv->lk_handle->reginfo.primary)->stat.st_maxobjects;
+		    env->lk_handle->reginfo.primary)->stat.st_maxobjects;
 	} else
 		*lk_maxp = dbenv->lk_max_objects;
 	return (0);
@@ -309,11 +361,59 @@ __lock_set_lk_max_objects(dbenv, lk_max)
 	DB_ENV *dbenv;
 	u_int32_t lk_max;
 {
-	ENV_ILLEGAL_AFTER_OPEN(dbenv, "DB_ENV->set_lk_max_objects");
+	ENV *env;
+
+	env = dbenv->env;
+
+	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_lk_max_objects");
 
 	dbenv->lk_max_objects = lk_max;
 	return (0);
 }
+/*
+ * PUBLIC: int __lock_get_lk_partitions __P((DB_ENV *, u_int32_t *));
+ */
+int
+__lock_get_lk_partitions(dbenv, lk_partitionp)
+	DB_ENV *dbenv;
+	u_int32_t *lk_partitionp;
+{
+	ENV *env;
+
+	env = dbenv->env;
+
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->get_lk_partitions", DB_INIT_LOCK);
+
+	if (LOCKING_ON(env)) {
+		/* Cannot be set after open, no lock required to read. */
+		*lk_partitionp = ((DB_LOCKREGION *)
+		    env->lk_handle->reginfo.primary)->stat.st_partitions;
+	} else
+		*lk_partitionp = dbenv->lk_partitions;
+	return (0);
+}
+
+/*
+ * __lock_set_lk_partitions
+ *	DB_ENV->set_lk_partitions.
+ *
+ * PUBLIC: int __lock_set_lk_partitions __P((DB_ENV *, u_int32_t));
+ */
+int
+__lock_set_lk_partitions(dbenv, lk_partitions)
+	DB_ENV *dbenv;
+	u_int32_t lk_partitions;
+{
+	ENV *env;
+
+	env = dbenv->env;
+
+	ENV_ILLEGAL_AFTER_OPEN(env, "DB_ENV->set_lk_partitions");
+
+	dbenv->lk_partitions = lk_partitions;
+	return (0);
+}
 
 /*
  * PUBLIC: int __lock_get_env_timeout
@@ -325,18 +425,23 @@ __lock_get_env_timeout(dbenv, timeoutp, flag)
 	db_timeout_t *timeoutp;
 	u_int32_t flag;
 {
-	DB_LOCKTAB *lt;
 	DB_LOCKREGION *region;
+	DB_LOCKTAB *lt;
+	DB_THREAD_INFO *ip;
+	ENV *env;
 	int ret;
 
-	ENV_NOT_CONFIGURED(dbenv,
-	    dbenv->lk_handle, "DB_ENV->get_env_timeout", DB_INIT_LOCK);
+	env = dbenv->env;
+
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->get_env_timeout", DB_INIT_LOCK);
 
 	ret = 0;
-	if (LOCKING_ON(dbenv)) {
-		lt = dbenv->lk_handle;
+	if (LOCKING_ON(env)) {
+		lt = env->lk_handle;
 		region = lt->reginfo.primary;
-		LOCK_REGION_LOCK(dbenv);
+		ENV_ENTER(env, ip);
+		LOCK_REGION_LOCK(env);
 		switch (flag) {
 		case DB_SET_LOCK_TIMEOUT:
 			*timeoutp = region->lk_timeout;
@@ -348,7 +453,8 @@ __lock_get_env_timeout(dbenv, timeoutp, flag)
 			ret = 1;
 			break;
 		}
-		LOCK_REGION_UNLOCK(dbenv);
+		LOCK_REGION_UNLOCK(env);
+		ENV_LEAVE(env, ip);
 	} else
 		switch (flag) {
 		case DB_SET_LOCK_TIMEOUT:
@@ -363,7 +469,7 @@ __lock_get_env_timeout(dbenv, timeoutp, flag)
 		}
 
 	if (ret)
-		ret = __db_ferr(dbenv, "DB_ENV->get_timeout", 0);
+		ret = __db_ferr(env, "DB_ENV->get_timeout", 0);
 
 	return (ret);
 }
@@ -380,18 +486,23 @@ __lock_set_env_timeout(dbenv, timeout, flags)
 	db_timeout_t timeout;
 	u_int32_t flags;
 {
-	DB_LOCKTAB *lt;
 	DB_LOCKREGION *region;
+	DB_LOCKTAB *lt;
+	DB_THREAD_INFO *ip;
+	ENV *env;
 	int ret;
 
-	ENV_NOT_CONFIGURED(dbenv,
-	    dbenv->lk_handle, "DB_ENV->set_env_timeout", DB_INIT_LOCK);
+	env = dbenv->env;
+
+	ENV_NOT_CONFIGURED(env,
+	    env->lk_handle, "DB_ENV->set_env_timeout", DB_INIT_LOCK);
 
 	ret = 0;
-	if (LOCKING_ON(dbenv)) {
-		lt = dbenv->lk_handle;
+	if (LOCKING_ON(env)) {
+		lt = env->lk_handle;
 		region = lt->reginfo.primary;
-		LOCK_REGION_LOCK(dbenv);
+		ENV_ENTER(env, ip);
+		LOCK_REGION_LOCK(env);
 		switch (flags) {
 		case DB_SET_LOCK_TIMEOUT:
 			region->lk_timeout = timeout;
@@ -403,7 +514,8 @@ __lock_set_env_timeout(dbenv, timeout, flags)
 			ret = 1;
 			break;
 		}
-		LOCK_REGION_UNLOCK(dbenv);
+		LOCK_REGION_UNLOCK(env);
+		ENV_LEAVE(env, ip);
 	} else
 		switch (flags) {
 		case DB_SET_LOCK_TIMEOUT:
@@ -418,7 +530,7 @@ __lock_set_env_timeout(dbenv, timeout, flags)
 		}
 
 	if (ret)
-		ret = __db_ferr(dbenv, "DB_ENV->set_timeout", 0);
+		ret = __db_ferr(env, "DB_ENV->set_timeout", 0);
 
 	return (ret);
 }
