@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997,2007 Oracle.  All rights reserved.
+ * Copyright (c) 1997,2008 Oracle.  All rights reserved.
  *
- * $Id: os_open.c,v 1.5 2007/05/17 15:15:47 bostic Exp $
+ * $Id: os_open.c,v 1.8 2008/03/26 04:11:34 david Exp $
  */
 
 #include "db_config.h"
@@ -15,8 +15,8 @@
  *	Open a file descriptor (including page size and log size information).
  */
 int
-__os_open(dbenv, name, page_size, flags, mode, fhpp)
-	DB_ENV *dbenv;
+__os_open(env, name, page_size, flags, mode, fhpp)
+	ENV *env;
 	const char *name;
 	u_int32_t page_size, flags;
 	int mode;
@@ -31,7 +31,7 @@ __os_open(dbenv, name, page_size, flags, mode, fhpp)
 	(DB_OSO_ABSMODE | DB_OSO_CREATE | DB_OSO_DIRECT | DB_OSO_DSYNC |\
 	DB_OSO_EXCL | DB_OSO_RDONLY | DB_OSO_REGION |	DB_OSO_SEQ |	\
 	DB_OSO_TEMP | DB_OSO_TRUNC)
-	if ((ret = __db_fchk(dbenv, "__os_open", flags, OKFLAGS)) != 0)
+	if ((ret = __db_fchk(env, "__os_open", flags, OKFLAGS)) != 0)
 		return (ret);
 
 	oflags = 0;
@@ -43,5 +43,9 @@ __os_open(dbenv, name, page_size, flags, mode, fhpp)
 	else
 		oflags |= _OFM_READWRITE;
 
-	return (__os_openhandle(dbenv, name, oflags, mode, fhpp));
+	if ((ret = __os_openhandle(env, name, oflags, mode, fhpp)) != 0)
+		return (ret);
+	if (LF_ISSET(DB_OSO_REGION))
+		F_SET(fhp, DB_FH_REGION);
+	return (ret);
 }
