@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2004,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2004,2008 Oracle.  All rights reserved.
  *
- * $Id: os_truncate.c,v 1.4 2007/05/17 15:15:47 bostic Exp $
+ * $Id: os_truncate.c,v 1.7 2008/02/18 19:34:22 bostic Exp $
  */
 
 #include "db_config.h"
@@ -15,8 +15,8 @@
  *	Truncate the file.
  */
 int
-__os_truncate(dbenv, fhp, pgno, pgsize)
-	DB_ENV *dbenv;
+__os_truncate(env, fhp, pgno, pgsize)
+	ENV *env;
 	DB_FH *fhp;
 	db_pgno_t pgno;
 	u_int32_t pgsize;
@@ -25,7 +25,7 @@ __os_truncate(dbenv, fhp, pgno, pgsize)
 	off_t offset;
 	int ret;
 
-	FILE_MANAGER_CREATE(dbenv, pIFileMgr, ret);
+	FILE_MANAGER_CREATE(env, pIFileMgr, ret);
 	if (ret != 0)
 		return (ret);
 
@@ -35,10 +35,12 @@ __os_truncate(dbenv, fhp, pgno, pgsize)
 	 */
 	offset = (off_t)pgsize * pgno;
 
+	LAST_PANIC_CHECK_BEFORE_IO(env);
+
 	if (IFILE_Truncate(fhp->ifp, offset) == SUCCESS)
 		ret = 0;
 	else
-		FILE_MANAGER_ERR(dbenv, pIFileMgr, NULL, "IFILE_Truncate", ret);
+		FILE_MANAGER_ERR(env, pIFileMgr, NULL, "IFILE_Truncate", ret);
 
 	IFILEMGR_Release(pIFileMgr);
 
