@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997,2007 Oracle.  All rights reserved.
+ * Copyright (c) 1997,2008 Oracle.  All rights reserved.
  *
- * $Id: os_fsync.c,v 12.10 2007/05/17 15:15:49 bostic Exp $
+ * $Id: os_fsync.c,v 12.13 2008/01/08 20:58:46 bostic Exp $
  */
 
 #include "db_config.h"
@@ -15,12 +15,15 @@
  *	Flush a file descriptor.
  */
 int
-__os_fsync(dbenv, fhp)
-	DB_ENV *dbenv;
+__os_fsync(env, fhp)
+	ENV *env;
 	DB_FH *fhp;
 {
+	DB_ENV *dbenv;
 	int ret;
 
+	dbenv = env == NULL ? NULL : env->dbenv;
+
 	/*
 	 * Do nothing if the file descriptor has been marked as not requiring
 	 * any sync to disk.
@@ -29,11 +32,11 @@ __os_fsync(dbenv, fhp)
 		return (0);
 
 	if (dbenv != NULL && FLD_ISSET(dbenv->verbose, DB_VERB_FILEOPS_ALL))
-		__db_msg(dbenv, "fileops: flush %s", fhp->name);
+		__db_msg(env, "fileops: flush %s", fhp->name);
 
 	RETRY_CHK((!FlushFileBuffers(fhp->handle)), ret);
 	if (ret != 0) {
-		__db_syserr(dbenv, ret, "FlushFileBuffers");
+		__db_syserr(env, ret, "FlushFileBuffers");
 		ret = __os_posix_err(ret);
 	}
 	return (ret);
