@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2005,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2005,2008 Oracle.  All rights reserved.
  *
- * $Id: repmgr_msg.c,v 1.36 2007/06/11 18:29:34 alanb Exp $
+ * $Id: repmgr_msg.c,v 1.42 2008/01/08 20:58:48 bostic Exp $
  */
 
 #include "db_config.h"
@@ -11,10 +11,10 @@
 #define	__INCLUDE_NETWORKING	1
 #include "db_int.h"
 
-static int message_loop __P((DB_ENV *));
-static int process_message __P((DB_ENV*, DBT*, DBT*, int));
-static int handle_newsite __P((DB_ENV *, const DBT *));
-static int ack_message __P((DB_ENV *, u_int32_t, DB_LSN *));
+static int message_loop __P((ENV *));
+static int process_message __P((ENV*, DBT*, DBT*, int));
+static int handle_newsite __P((ENV *, const DBT *));
+static int ack_message __P((ENV *, u_int32_t, DB_LSN *));
 
 /*
  * PUBLIC: void *__repmgr_msg_thread __P((void *));
@@ -23,29 +23,30 @@ void *
 __repmgr_msg_thread(args)
 	void *args;
 {
-	DB_ENV *dbenv = args;
+	ENV *env = args;
 	int ret;
 
-	if ((ret = message_loop(dbenv)) != 0) {
-		__db_err(dbenv, ret, "message thread failed");
-		__repmgr_thread_failure(dbenv, ret);
+	if ((ret = message_loop(env)) != 0) {
+		__db_err(env, ret, "message thread failed");
+		__repmgr_thread_failure(env, ret);
 	}
 	return (NULL);
 }
 
 static int
-message_loop(dbenv)
-	DB_ENV *dbenv;
+message_loop(env)
+	ENV *env;
 {
 	REPMGR_MESSAGE *msg;
 	int ret;
 
-	while ((ret = __repmgr_queue_get(dbenv, &msg)) == 0) {
-		while ((ret = process_message(dbenv, &msg->control, &msg->rec,
+	while ((ret = __repmgr_queue_get(env, &msg)) == 0) {
+		while ((ret = process_message(env, &msg->control, &msg->rec,
 		    msg->originating_eid)) == DB_LOCK_DEADLOCK)
-			RPRINT(dbenv, (dbenv, "repmgr deadlock retry"));
+			RPRINT(env, DB_VERB_REPMGR_MISC,
+			    (env, "repmgr deadlock retry"));
 
-		__os_free(dbenv, msg);
+		__os_free(env, msg);
 		if (ret != 0)
 			return (ret);
 	}
@@ -54,18 +55,18 @@ message_loop(dbenv)
 }
 
 static int
-process_message(dbenv, control, rec, eid)
-	DB_ENV *dbenv;
+process_message(env, control, rec, eid)
+	ENV *env;
 	DBT *control, *rec;
 	int eid;
 {
+	DB_LSN permlsn;
 	DB_REP *db_rep;
 	REP *rep;
-	DB_LSN permlsn;
 	int ret;
 	u_int32_t generation;
 
-	db_rep = dbenv->rep_handle;
+	db_rep = env->rep_handle;
 
 	/*
 	 * Save initial generation number, in case it changes in a close race
@@ -74,30 +75,30 @@ process_message(dbenv, control, rec, eid)
 	generation = db_rep->generation;
 
 	switch (ret =
-	    __rep_process_message(dbenv, control, rec, eid, &permlsn)) {
+	    __rep_process_message(env->dbenv, control, rec, eid, &permlsn)) {
 	case 0:
 		if (db_rep->takeover_pending) {
 			db_rep->takeover_pending = FALSE;
-			return (__repmgr_become_master(dbenv));
+			return (__repmgr_become_master(env));
 		}
 		break;
 
 	case DB_REP_NEWSITE:
-		return (handle_newsite(dbenv, rec));
+		return (handle_newsite(env, rec));
 
 	case DB_REP_HOLDELECTION:
 		LOCK_MUTEX(db_rep->mutex);
-		ret = __repmgr_init_election(dbenv, ELECT_ELECTION);
+		ret = __repmgr_init_election(env, ELECT_ELECTION);
 		UNLOCK_MUTEX(db_rep->mutex);
 		if (ret != 0)
 			return (ret);
 		break;
 
 	case DB_REP_DUPMASTER:
-		if ((ret = __repmgr_repstart(dbenv, DB_REP_CLIENT)) != 0)
+		if ((ret = __repmgr_repstart(env, DB_REP_CLIENT)) != 0)
 			return (ret);
 		LOCK_MUTEX(db_rep->mutex);
-		ret = __repmgr_init_election(dbenv, ELECT_ELECTION);
+		ret = __repmgr_init_election(env, ELECT_ELECTION);
 		UNLOCK_MUTEX(db_rep->mutex);
 		if (ret != 0)
 			return (ret);
@@ -113,7 +114,7 @@ process_message(dbenv, control, rec, eid)
 		    rep->priority == 0))
 			break;
 
-		if ((ret = ack_message(dbenv, generation, &permlsn)) != 0)
+		if ((ret = ack_message(env, generation, &permlsn)) != 0)
 			return (ret);
 
 		break;
@@ -124,7 +125,7 @@ process_message(dbenv, control, rec, eid)
 		break;
 
 	default:
-		__db_err(dbenv, ret, "DB_ENV->rep_process_message");
+		__db_err(env, ret, "DB_ENV->rep_process_message");
 		return (ret);
 	}
 	return (0);
@@ -134,17 +135,17 @@ process_message(dbenv, control, rec, eid)
  * Handle replication-related events.  Returns only 0 or DB_EVENT_NOT_HANDLED;
  * no other error returns are tolerated.
  *
- * PUBLIC: int __repmgr_handle_event __P((DB_ENV *, u_int32_t, void *));
+ * PUBLIC: int __repmgr_handle_event __P((ENV *, u_int32_t, void *));
  */
 int
-__repmgr_handle_event(dbenv, event, info)
-	DB_ENV *dbenv;
+__repmgr_handle_event(env, event, info)
+	ENV *env;
 	u_int32_t event;
 	void *info;
 {
 	DB_REP *db_rep;
 
-	db_rep = dbenv->rep_handle;
+	db_rep = env->rep_handle;
 
 	if (db_rep->selector == NULL) {
 		/* Repmgr is not in use, so all events go to application. */
@@ -153,7 +154,7 @@ __repmgr_handle_event(dbenv, event, info)
 
 	switch (event) {
 	case DB_EVENT_REP_ELECTED:
-		DB_ASSERT(dbenv, info == NULL);
+		DB_ASSERT(env, info == NULL);
 
 		db_rep->found_master = TRUE;
 		db_rep->takeover_pending = TRUE;
@@ -167,11 +168,11 @@ __repmgr_handle_event(dbenv, event, info)
 		 */
 		break;
 	case DB_EVENT_REP_NEWMASTER:
-		DB_ASSERT(dbenv, info != NULL);
+		DB_ASSERT(env, info != NULL);
 
 		db_rep->found_master = TRUE;
 		db_rep->master_eid = *(int *)info;
-		__repmgr_stash_generation(dbenv);
+		__repmgr_stash_generation(env);
 
 		/* Application still needs to see this. */
 		break;
@@ -183,25 +184,22 @@ __repmgr_handle_event(dbenv, event, info)
 
 /*
  * Acknowledges a message.
- *
- * !!!
- * Note that this cannot be called from the select() thread, in case we call
- * __repmgr_bust_connection(..., FALSE).
  */
 static int
-ack_message(dbenv, generation, lsn)
-	DB_ENV *dbenv;
+ack_message(env, generation, lsn)
+	ENV *env;
 	u_int32_t generation;
 	DB_LSN *lsn;
 {
-	DB_REP *db_rep;
-	REPMGR_SITE *site;
-	REPMGR_CONNECTION *conn;
-	DB_REPMGR_ACK ack;
 	DBT control2, rec2;
+	DB_REP *db_rep;
+	__repmgr_ack_args ack;
+	u_int8_t buf[__REPMGR_ACK_SIZE];
+	REPMGR_CONNECTION *conn;
+	REPMGR_SITE *site;
 	int ret;
 
-	db_rep = dbenv->rep_handle;
+	db_rep = env->rep_handle;
 	/*
 	 * Regardless of where a message came from, all ack's go to the master
 	 * site.  If we're not in touch with the master, we drop it, since
@@ -209,7 +207,7 @@ ack_message(dbenv, generation, lsn)
 	 */
 	if (!IS_VALID_EID(db_rep->master_eid) ||
 	    db_rep->master_eid == SELF_EID) {
-		RPRINT(dbenv, (dbenv,
+		RPRINT(env, DB_VERB_REPMGR_MISC, (env,
 		    "dropping ack with master %d", db_rep->master_eid));
 		return (0);
 	}
@@ -218,18 +216,28 @@ ack_message(dbenv, generation, lsn)
 	LOCK_MUTEX(db_rep->mutex);
 	site = SITE_FROM_EID(db_rep->master_eid);
 	if (site->state == SITE_CONNECTED &&
-	    !F_ISSET(site->ref.conn, CONN_CONNECTING)) {
-
+	    site->ref.conn->state == CONN_READY) {
+		conn = site->ref.conn;
+		DB_ASSERT(env, conn->version > 0);
 		ack.generation = generation;
 		memcpy(&ack.lsn, lsn, sizeof(DB_LSN));
-		control2.data = &ack;
-		control2.size = sizeof(ack);
+		if (conn->version == 1) {
+			control2.data = &ack;
+			control2.size = sizeof(ack);
+		} else {
+			__repmgr_ack_marshal(env, &ack, buf);
+			control2.data = buf;
+			control2.size = __REPMGR_ACK_SIZE;
+		}
 		rec2.size = 0;
-
-		conn = site->ref.conn;
-		if ((ret = __repmgr_send_one(dbenv, conn, REPMGR_ACK,
-		    &control2, &rec2)) == DB_REP_UNAVAIL)
-			ret = __repmgr_bust_connection(dbenv, conn, FALSE);
+		/*
+		 * It's hard to imagine anyone would care about a lost ack if
+		 * the path to the master is so congested as to need blocking;
+		 * so pass "blockable" argument as FALSE.
+		 */
+		if ((ret = __repmgr_send_one(env, conn, REPMGR_ACK,
+		    &control2, &rec2, FALSE)) == DB_REP_UNAVAIL)
+			ret = __repmgr_bust_connection(env, conn);
 	}
 
 	UNLOCK_MUTEX(db_rep->mutex);
@@ -240,8 +248,8 @@ ack_message(dbenv, generation, lsn)
  * Does everything necessary to handle the processing of a NEWSITE return.
  */
 static int
-handle_newsite(dbenv, rec)
-	DB_ENV *dbenv;
+handle_newsite(env, rec)
+	ENV *env;
 	const DBT *rec;
 {
 	ADDRINFO *ai;
@@ -254,7 +262,7 @@ handle_newsite(dbenv, rec)
 	int ret;
 	char *host;
 
-	db_rep = dbenv->rep_handle;
+	db_rep = env->rep_handle;
 	/*
 	 * Check if we got sent connect information and if we did, if
 	 * this is me or if we already have a connection to this new
@@ -265,7 +273,7 @@ handle_newsite(dbenv, rec)
 	 * null-terminated, but let's make sure.
 	 */
 	if (rec->size < sizeof(port) + 1) {
-		__db_errx(dbenv, "unexpected cdata size, msg ignored");
+		__db_errx(env, "unexpected cdata size, msg ignored");
 		return (0);
 	}
 	memcpy(&port, rec->data, sizeof(port));
@@ -278,13 +286,14 @@ handle_newsite(dbenv, rec)
 	/* It's me, do nothing. */
 	if (strcmp(host, db_rep->my_addr.host) == 0 &&
 	    port == db_rep->my_addr.port) {
-		RPRINT(dbenv, (dbenv, "repmgr ignores own NEWSITE info"));
+		RPRINT(env, DB_VERB_REPMGR_MISC,
+		    (env, "repmgr ignores own NEWSITE info"));
 		return (0);
 	}
 
 	LOCK_MUTEX(db_rep->mutex);
-	if ((ret = __repmgr_add_site(dbenv, host, port, &site)) == EEXIST) {
-		RPRINT(dbenv, (dbenv,
+	if ((ret = __repmgr_add_site(env, host, port, &site)) == EEXIST) {
+		RPRINT(env, DB_VERB_REPMGR_MISC, (env,
 		    "NEWSITE info from %s was already known",
 		    __repmgr_format_site_loc(site, buffer)));
 		/*
@@ -297,7 +306,7 @@ handle_newsite(dbenv, rec)
 		 */
 		addr = &site->net_addr;
 		if (addr->address_list == NULL) {
-			if ((ret = __repmgr_getaddr(dbenv,
+			if ((ret = __repmgr_getaddr(env,
 			    addr->host, addr->port, 0, &ai)) == 0)
 				addr->address_list = ai;
 			else if (ret != DB_REP_UNAVAIL)
@@ -310,7 +319,8 @@ handle_newsite(dbenv, rec)
 	} else {
 		if (ret != 0)
 			goto unlock;
-		RPRINT(dbenv, (dbenv, "NEWSITE info added %s",
+		RPRINT(env, DB_VERB_REPMGR_MISC,
+		    (env, "NEWSITE info added %s",
 		    __repmgr_format_site_loc(site, buffer)));
 	}
 
@@ -318,23 +328,23 @@ handle_newsite(dbenv, rec)
 	 * Wake up the main thread to connect to the new or reawakened
 	 * site.
 	 */
-	ret = __repmgr_wake_main_thread(dbenv);
+	ret = __repmgr_wake_main_thread(env);
 
 unlock: UNLOCK_MUTEX(db_rep->mutex);
 	return (ret);
 }
 
 /*
- * PUBLIC: void __repmgr_stash_generation __P((DB_ENV *));
+ * PUBLIC: void __repmgr_stash_generation __P((ENV *));
  */
 void
-__repmgr_stash_generation(dbenv)
-	DB_ENV *dbenv;
+__repmgr_stash_generation(env)
+	ENV *env;
 {
 	DB_REP *db_rep;
 	REP *rep;
 
-	db_rep = dbenv->rep_handle;
+	db_rep = env->rep_handle;
 	rep = db_rep->region;
 
 	db_rep->generation = rep->gen;
