@@ -1,23 +1,17 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1999-2004
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 1999,2008 Oracle.  All rights reserved.
  *
- * $Id: tcl_mp.c,v 11.58 2004/10/07 16:48:39 bostic Exp $
+ * $Id: tcl_mp.c,v 12.18 2008/03/28 01:16:02 mbrey Exp $
  */
 
 #include "db_config.h"
 
-#ifndef NO_SYSTEM_INCLUDES
-#include <sys/types.h>
-
-#include <stdlib.h>
-#include <string.h>
+#include "db_int.h"
+#ifdef HAVE_SYSTEM_INCLUDE_FILES
 #include <tcl.h>
 #endif
-
-#include "db_int.h"
 #include "dbinc/tcl_db.h"
 
 /*
@@ -29,7 +23,7 @@ static int      pg_Cmd __P((ClientData, Tcl_Interp *, int, Tcl_Obj * CONST*));
 static int      tcl_MpGet __P((Tcl_Interp *, int, Tcl_Obj * CONST*,
     DB_MPOOLFILE *, DBTCL_INFO *));
 static int      tcl_Pg __P((Tcl_Interp *, int, Tcl_Obj * CONST*,
-    void *, DB_MPOOLFILE *, DBTCL_INFO *, int));
+    void *, DB_MPOOLFILE *, DBTCL_INFO *));
 static int      tcl_PgInit __P((Tcl_Interp *, int, Tcl_Obj * CONST*,
     void *, DBTCL_INFO *));
 static int      tcl_PgIsset __P((Tcl_Interp *, int, Tcl_Obj * CONST*,
@@ -70,11 +64,11 @@ _MpInfoDelete(interp, mpip)
  * PUBLIC: int tcl_MpSync __P((Tcl_Interp *, int, Tcl_Obj * CONST*, DB_ENV *));
  */
 int
-tcl_MpSync(interp, objc, objv, envp)
+tcl_MpSync(interp, objc, objv, dbenv)
 	Tcl_Interp *interp;		/* Interpreter */
 	int objc;			/* How many arguments? */
 	Tcl_Obj *CONST objv[];		/* The argument objects */
-	DB_ENV *envp;			/* Environment pointer */
+	DB_ENV *dbenv;			/* Environment pointer */
 {
 
 	DB_LSN lsn, *lsnp;
@@ -97,7 +91,7 @@ tcl_MpSync(interp, objc, objv, envp)
 	}
 
 	_debug_check();
-	ret = envp->memp_sync(envp, lsnp);
+	ret = dbenv->memp_sync(dbenv, lsnp);
 	return (_ReturnSetup(interp, ret, DB_RETOK_STD(ret), "memp sync"));
 }
 
@@ -108,11 +102,11 @@ tcl_MpSync(interp, objc, objv, envp)
  * PUBLIC:    Tcl_Obj * CONST*, DB_ENV *));
  */
 int
-tcl_MpTrickle(interp, objc, objv, envp)
+tcl_MpTrickle(interp, objc, objv, dbenv)
 	Tcl_Interp *interp;		/* Interpreter */
 	int objc;			/* How many arguments? */
 	Tcl_Obj *CONST objv[];		/* The argument objects */
-	DB_ENV *envp;			/* Environment pointer */
+	DB_ENV *dbenv;			/* Environment pointer */
 {
 
 	Tcl_Obj *res;
@@ -132,7 +126,7 @@ tcl_MpTrickle(interp, objc, objv, envp)
 		return (result);
 
 	_debug_check();
-	ret = envp->memp_trickle(envp, percent, &pages);
+	ret = dbenv->memp_trickle(dbenv, percent, &pages);
 	result = _ReturnSetup(interp, ret, DB_RETOK_STD(ret), "memp trickle");
 	if (result == TCL_ERROR)
 		return (result);
@@ -150,16 +144,17 @@ tcl_MpTrickle(interp, objc, objv, envp)
  * PUBLIC:    Tcl_Obj * CONST*, DB_ENV *, DBTCL_INFO *));
  */
 int
-tcl_Mp(interp, objc, objv, envp, envip)
+tcl_Mp(interp, objc, objv, dbenv, envip)
 	Tcl_Interp *interp;		/* Interpreter */
 	int objc;			/* How many arguments? */
 	Tcl_Obj *CONST objv[];		/* The argument objects */
-	DB_ENV *envp;			/* Environment pointer */
+	DB_ENV *dbenv;			/* Environment pointer */
 	DBTCL_INFO *envip;		/* Info pointer */
 {
 	static const char *mpopts[] = {
 		"-create",
 		"-mode",
+		"-multiversion",
 		"-nommap",
 		"-pagesize",
 		"-rdonly",
@@ -168,6 +163,7 @@ tcl_Mp(interp, objc, objv, envp, envip)
 	enum mpopts {
 		MPCREATE,
 		MPMODE,
+		MPMULTIVERSION,
 		MPNOMMAP,
 		MPPAGE,
 		MPRDONLY
@@ -203,6 +199,24 @@ tcl_Mp(interp, objc, objv, envp, envip)
 		case MPCREATE:
 			flag |= DB_CREATE;
 			break;
+		case MPMODE:
+			if (i >= objc) {
+				Tcl_WrongNumArgs(interp, 2, objv,
+				    "?-mode mode?");
+				result = TCL_ERROR;
+				break;
+			}
+			/*
+			 * Don't need to check result here because
+			 * if TCL_ERROR, the error message is already
+			 * set up, and we'll bail out below.  If ok,
+			 * the mode is set and we go on.
+			 */
+			result = Tcl_GetIntFromObj(interp, objv[i++], &mode);
+			break;
+		case MPMULTIVERSION:
+			flag |= DB_MULTIVERSION;
+			break;
 		case MPNOMMAP:
 			flag |= DB_NOMMAP;
 			break;
@@ -224,21 +238,6 @@ tcl_Mp(interp, objc, objv, envp, envip)
 		case MPRDONLY:
 			flag |= DB_RDONLY;
 			break;
-		case MPMODE:
-			if (i >= objc) {
-				Tcl_WrongNumArgs(interp, 2, objv,
-				    "?-mode mode?");
-				result = TCL_ERROR;
-				break;
-			}
-			/*
-			 * Don't need to check result here because
-			 * if TCL_ERROR, the error message is already
-			 * set up, and we'll bail out below.  If ok,
-			 * the mode is set and we go on.
-			 */
-			result = Tcl_GetIntFromObj(interp, objv[i++], &mode);
-			break;
 		}
 		if (result != TCL_OK)
 			goto error;
@@ -266,7 +265,7 @@ tcl_Mp(interp, objc, objv, envp, envip)
 	}
 
 	_debug_check();
-	if ((ret = envp->memp_fcreate(envp, &mpf, 0)) != 0) {
+	if ((ret = dbenv->memp_fcreate(dbenv, &mpf, 0)) != 0) {
 		result = _ReturnSetup(interp, ret, DB_RETOK_STD(ret), "mpool");
 		_DeleteInfo(ip);
 		goto error;
@@ -307,14 +306,14 @@ error:
  * PUBLIC: int tcl_MpStat __P((Tcl_Interp *, int, Tcl_Obj * CONST*, DB_ENV *));
  */
 int
-tcl_MpStat(interp, objc, objv, envp)
+tcl_MpStat(interp, objc, objv, dbenv)
 	Tcl_Interp *interp;		/* Interpreter */
 	int objc;			/* How many arguments? */
 	Tcl_Obj *CONST objv[];		/* The argument objects */
-	DB_ENV *envp;			/* Environment pointer */
+	DB_ENV *dbenv;			/* Environment pointer */
 {
-	DB_MPOOL_STAT *sp;
 	DB_MPOOL_FSTAT **fsp, **savefsp;
+	DB_MPOOL_STAT *sp;
 	int result;
 	int ret;
 	Tcl_Obj *res;
@@ -330,7 +329,7 @@ tcl_MpStat(interp, objc, objv, envp)
 		return (TCL_ERROR);
 	}
 	_debug_check();
-	ret = envp->memp_stat(envp, &sp, &fsp, 0);
+	ret = dbenv->memp_stat(dbenv, &sp, &fsp, 0);
 	result = _ReturnSetup(interp, ret, DB_RETOK_STD(ret), "memp stat");
 	if (result == TCL_ERROR)
 		return (result);
@@ -340,12 +339,14 @@ tcl_MpStat(interp, objc, objv, envp)
 	 * list pairs and free up the memory.
 	 */
 	res = Tcl_NewObj();
+#ifdef HAVE_STATISTICS
 	/*
 	 * MAKE_STAT_LIST assumes 'res' and 'error' label.
 	 */
 	MAKE_STAT_LIST("Cache size (gbytes)", sp->st_gbytes);
 	MAKE_STAT_LIST("Cache size (bytes)", sp->st_bytes);
 	MAKE_STAT_LIST("Number of caches", sp->st_ncache);
+	MAKE_STAT_LIST("Maximum number of caches", sp->st_max_ncache);
 	MAKE_STAT_LIST("Region size", sp->st_regsize);
 	MAKE_STAT_LIST("Maximum memory-mapped file size", sp->st_mmapsize);
 	MAKE_STAT_LIST("Maximum open file descriptors", sp->st_maxopenfd);
@@ -370,10 +371,15 @@ tcl_MpStat(interp, objc, objv, envp)
 	MAKE_STAT_LIST("Hash elements examined", sp->st_hash_examined);
 	MAKE_STAT_LIST("Number of hash bucket nowaits", sp->st_hash_nowait);
 	MAKE_STAT_LIST("Number of hash bucket waits", sp->st_hash_wait);
+	MAKE_STAT_LIST("Maximum number of hash bucket nowaits",
+	    sp->st_hash_max_nowait);
 	MAKE_STAT_LIST("Maximum number of hash bucket waits",
 	    sp->st_hash_max_wait);
 	MAKE_STAT_LIST("Number of region lock nowaits", sp->st_region_nowait);
 	MAKE_STAT_LIST("Number of region lock waits", sp->st_region_wait);
+	MAKE_STAT_LIST("Buffers frozen", sp->st_mvcc_frozen);
+	MAKE_STAT_LIST("Buffers thawed", sp->st_mvcc_thawed);
+	MAKE_STAT_LIST("Frozen buffers freed", sp->st_mvcc_freed);
 	MAKE_STAT_LIST("Page allocations", sp->st_alloc);
 	MAKE_STAT_LIST("Buckets examined during allocation",
 	    sp->st_alloc_buckets);
@@ -382,6 +388,7 @@ tcl_MpStat(interp, objc, objv, envp)
 	MAKE_STAT_LIST("Pages examined during allocation", sp->st_alloc_pages);
 	MAKE_STAT_LIST("Maximum pages examined during allocation",
 	    sp->st_alloc_max_pages);
+	MAKE_STAT_LIST("Threads waiting on buffer I/O", sp->st_io_wait);
 
 	/*
 	 * Save global stat list as res1.  The MAKE_STAT_LIST
@@ -412,11 +419,12 @@ tcl_MpStat(interp, objc, objv, envp)
 		if (result != TCL_OK)
 			goto error;
 	}
+#endif
 	Tcl_SetObjResult(interp, res1);
 error:
-	__os_ufree(envp, sp);
+	__os_ufree(dbenv->env, sp);
 	if (savefsp != NULL)
-		__os_ufree(envp, savefsp);
+		__os_ufree(dbenv->env, savefsp);
 	return (result);
 }
 
@@ -587,24 +595,30 @@ tcl_MpGet(interp, objc, objv, mp, mpip)
 {
 	static const char *mpget[] = {
 		"-create",
+		"-dirty",
 		"-last",
 		"-new",
+		"-txn",
 		NULL
 	};
 	enum mpget {
 		MPGET_CREATE,
+		MPGET_DIRTY,
 		MPGET_LAST,
-		MPGET_NEW
+		MPGET_NEW,
+		MPGET_TXN
 	};
 
 	DBTCL_INFO *ip;
 	Tcl_Obj *res;
+	DB_TXN *txn;
 	db_pgno_t pgno;
 	u_int32_t flag;
 	int i, ipgno, optindex, result, ret;
-	char newname[MSG_SIZE];
+	char *arg, msg[MSG_SIZE], newname[MSG_SIZE];
 	void *page;
 
+	txn = NULL;
 	result = TCL_OK;
 	memset(newname, 0, MSG_SIZE);
 	i = 2;
@@ -627,12 +641,30 @@ tcl_MpGet(interp, objc, objv, mp, mpip)
 		case MPGET_CREATE:
 			flag |= DB_MPOOL_CREATE;
 			break;
+		case MPGET_DIRTY:
+			flag |= DB_MPOOL_DIRTY;
+			break;
 		case MPGET_LAST:
 			flag |= DB_MPOOL_LAST;
 			break;
 		case MPGET_NEW:
 			flag |= DB_MPOOL_NEW;
 			break;
+		case MPGET_TXN:
+			if (i == objc) {
+				Tcl_WrongNumArgs(interp, 2, objv, "?-txn id?");
+				result = TCL_ERROR;
+				break;
+			}
+			arg = Tcl_GetStringFromObj(objv[i++], NULL);
+			txn = NAME_TO_TXN(arg);
+			if (txn == NULL) {
+				snprintf(msg, MSG_SIZE,
+				    "mpool get: Invalid txn: %s\n", arg);
+				Tcl_SetResult(interp, msg, TCL_VOLATILE);
+				result = TCL_ERROR;
+			}
+			break;
 		}
 		if (result != TCL_OK)
 			goto error;
@@ -662,7 +694,7 @@ tcl_MpGet(interp, objc, objv, mp, mpip)
 	}
 	_debug_check();
 	pgno = (db_pgno_t)ipgno;
-	ret = mp->get(mp, &pgno, flag, &page);
+	ret = mp->get(mp, &pgno, NULL, flag, &page);
 	result = _ReturnSetup(interp, ret, DB_RETOK_MPGET(ret), "mpool get");
 	if (result == TCL_ERROR)
 		_DeleteInfo(ip);
@@ -702,7 +734,6 @@ pg_Cmd(clientData, interp, objc, objv)
 		"pgnum",
 		"pgsize",
 		"put",
-		"set",
 		NULL
 	};
 	enum pgcmds {
@@ -710,8 +741,7 @@ pg_Cmd(clientData, interp, objc, objv)
 		PGISSET,
 		PGNUM,
 		PGSIZE,
-		PGPUT,
-		PGSET
+		PGPUT
 	};
 	DB_MPOOLFILE *mp;
 	int cmdindex, length, result;
@@ -756,10 +786,8 @@ pg_Cmd(clientData, interp, objc, objv)
 	case PGSIZE:
 		res = Tcl_NewWideIntObj((Tcl_WideInt)pgip->i_pgsz);
 		break;
-	case PGSET:
 	case PGPUT:
-		result = tcl_Pg(interp, objc, objv, page, mp, pgip,
-		    (enum pgcmds)cmdindex == PGSET ? 0 : 1);
+		result = tcl_Pg(interp, objc, objv, page, mp, pgip);
 		break;
 	case PGINIT:
 		result = tcl_PgInit(interp, objc, objv, page, pgip);
@@ -779,24 +807,19 @@ pg_Cmd(clientData, interp, objc, objv)
 }
 
 static int
-tcl_Pg(interp, objc, objv, page, mp, pgip, putop)
+tcl_Pg(interp, objc, objv, page, mp, pgip)
 	Tcl_Interp *interp;		/* Interpreter */
 	int objc;			/* How many arguments? */
 	Tcl_Obj *CONST objv[];		/* The argument objects */
 	void *page;			/* Page pointer */
 	DB_MPOOLFILE *mp;		/* Mpool pointer */
 	DBTCL_INFO *pgip;		/* Info pointer */
-	int putop;			/* Operation */
 {
 	static const char *pgopt[] = {
-		"-clean",
-		"-dirty",
 		"-discard",
 		NULL
 	};
 	enum pgopt {
-		PGCLEAN,
-		PGDIRTY,
 		PGDISCARD
 	};
 	u_int32_t flag;
@@ -811,12 +834,6 @@ tcl_Pg(interp, objc, objv, page, mp, pgip, putop)
 			return (IS_HELP(objv[i]));
 		i++;
 		switch ((enum pgopt)optindex) {
-		case PGCLEAN:
-			flag |= DB_MPOOL_CLEAN;
-			break;
-		case PGDIRTY:
-			flag |= DB_MPOOL_DIRTY;
-			break;
 		case PGDISCARD:
 			flag |= DB_MPOOL_DISCARD;
 			break;
@@ -824,17 +841,12 @@ tcl_Pg(interp, objc, objv, page, mp, pgip, putop)
 	}
 
 	_debug_check();
-	if (putop)
-		ret = mp->put(mp, page, flag);
-	else
-		ret = mp->set(mp, page, flag);
+	ret = mp->put(mp, page, DB_PRIORITY_UNCHANGED, flag);
 
 	result = _ReturnSetup(interp, ret, DB_RETOK_STD(ret), "page");
 
-	if (putop) {
-		(void)Tcl_DeleteCommand(interp, pgip->i_name);
-		_DeleteInfo(pgip);
-	}
+	(void)Tcl_DeleteCommand(interp, pgip->i_name);
+	_DeleteInfo(pgip);
 	return (result);
 }
 
