@@ -1,8 +1,8 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2005,2007 Oracle.  All rights reserved.
+# Copyright (c) 2005,2008 Oracle.  All rights reserved.
 #
-# $Id: rep044.tcl,v 12.14 2007/05/17 18:17:21 bostic Exp $
+# $Id: rep044.tcl,v 12.16 2008/01/08 20:58:53 bostic Exp $
 #
 # TEST	rep044
 # TEST
@@ -52,10 +52,11 @@ proc rep044_sub { method tnum logset largs } {
 	set orig_tdir $testdir
 
 	global rep_verbose
+	global verbose_type
 
 	set verbargs ""
 	if { $rep_verbose == 1 } {
-		set verbargs " -verbose {rep on} "
+		set verbargs " -verbose {$verbose_type on} "
 	}
 
 	set masterdir $testdir/ENV0
